@@ -4,84 +4,93 @@ from django.forms import TextInput
 from tests.base import BootstrapTestCase
 
 
+class CharFieldTestForm(forms.Form):
+    test = forms.CharField()
+
+
 class InputTypeTextTestCase(BootstrapTestCase):
-    """Test for TextInput widgets that only differ in type, and have no other exceptions."""
-
-    def _html_default(self, input_type):
-        """Output for input with given input_type in default layout."""
-        return (
-            '<div class="django_bootstrap5-req mb-3">'
-            '<label for="id_test" class="form-label">Test</label>'
-            f'<input class="form-control" id="id_test" name="test" placeholder="Test" required type="{input_type}">'
-            "</div>"
-        )
-
-    def _html_horizontal(self, input_type):
-        """Output for input with given input_type in horizontal layout."""
-        return (
-            '<div class="django_bootstrap5-req mb-3 row">'
-            '<label class="col-form-label col-sm-2" for="id_test">'
-            "Test"
-            '</label><div class="col-sm-10">'
-            f'<input class="form-control" id="id_test" name="test" placeholder="Test" required type="{input_type}">'
-            "</div>"
-            "</div>"
-        )
-
-    def _html_floating(self, input_type):
-        """
-        Output for input with given input_type in floating layout.
-
-        Note that this function does not check if the flaoting layout is supported for this input_type.
-        """
-        return (
-            '<div class="django_bootstrap5-req mb-3 form-floating">'
-            f'<input class="form-control" id="id_test" name="test" placeholder="Test" required type="{input_type}">'
-            '<label for="id_test" class="form-label">Test</label>'
-            "</div>"
-        )
+    """Test for TextInput widgets that only differ in `input_type`."""
 
     def test_input_type_text(self):
         """Test field with default CharField widget."""
 
-        class TextTestForm(forms.Form):
-            test = forms.CharField()
-
-        form = TextTestForm()
+        form = CharFieldTestForm()
 
         self.assertHTMLEqual(
             self.render("{% bootstrap_field form.test %}", context={"form": form}),
-            self._html_default("text"),
+            (
+                '<div class="django_bootstrap5-req mb-3">'
+                '<label for="id_test" class="form-label">Test</label>'
+                '<input class="form-control" id="id_test" name="test" placeholder="Test" required type="text">'
+                "</div>"
+            ),
         )
 
-        addon_html = (
-            '<div class="django_bootstrap5-req mb-3">'
-            '<label for="id_test" class="form-label">Test</label>'
-            '<div class="input-group mb-3">'
-            '<span class="input-group-text">foo</span>'
-            '<input class="form-control" id="id_test" name="test" placeholder="Test" required type="text">'
-            "</div>"
-            "</div>"
+        form = CharFieldTestForm(data={})
+        self.assertHTMLEqual(
+            self.render("{% bootstrap_field form.test %}", context={"form": form}),
+            (
+                '<div class="django_bootstrap5-err django_bootstrap5-req mb-3">'
+                '<label class="form-label" for="id_test">Test</label>'
+                '<input type="text" name="test"'
+                ' class="form-control is-invalid" placeholder="Test" required id="id_test">'
+                '<div class="invalid-feedback">This field is required.</div>'
+                "</div>"
+            ),
         )
 
+    def test_input_type_text_more(self):
+        """Test field with default CharField widget."""
+
+        form = CharFieldTestForm()
+
         self.assertHTMLEqual(
             self.render('{% bootstrap_field form.test addon_before="foo" %}', context={"form": form}),
-            addon_html,
+            (
+                '<div class="django_bootstrap5-req mb-3">'
+                '<label for="id_test" class="form-label">Test</label>'
+                '<div class="input-group">'
+                '<span class="input-group-text">foo</span>'
+                '<input class="form-control" id="id_test" name="test" placeholder="Test" required type="text">'
+                "</div>"
+                "</div>"
+            ),
         )
 
         self.assertHTMLEqual(
             self.render('{% bootstrap_field form.test addon_before="foo" layout="floating" %}', context={"form": form}),
-            addon_html,
+            (
+                '<div class="django_bootstrap5-req mb-3">'
+                '<label for="id_test" class="form-label">Test</label>'
+                '<div class="input-group">'
+                '<span class="input-group-text">foo</span>'
+                '<input class="form-control" id="id_test" name="test" placeholder="Test" required type="text">'
+                "</div>"
+                "</div>"
+            ),
         )
 
         self.assertHTMLEqual(
             self.render("{% bootstrap_field form.test layout='horizontal' %}", context={"form": form}),
-            self._html_horizontal("text"),
+            (
+                '<div class="django_bootstrap5-req mb-3 row">'
+                '<label class="col-form-label col-sm-2" for="id_test">'
+                "Test"
+                '</label><div class="col-sm-10">'
+                '<input class="form-control" id="id_test" name="test" placeholder="Test" required type="text">'
+                "</div>"
+                "</div>"
+            ),
         ),
 
         self.assertHTMLEqual(
             self.render("{% bootstrap_field form.test layout='floating' %}", context={"form": form}),
-            self._html_floating("text"),
+            (
+                '<div class="django_bootstrap5-req mb-3 form-floating">'
+                '<input class="form-control" id="id_test" name="test" placeholder="Test" required type="text">'
+                '<label for="id_test" class="form-label">Test</label>'
+                "</div>"
+            ),
         )
 
     def _test_input_type(self, input_type):
@@ -91,9 +100,27 @@ class InputTypeTextTestCase(BootstrapTestCase):
             test = forms.CharField(widget=TextInput(attrs={"type": input_type}))
 
         form = InputTypeTestForm()
-        default_html = self._html_default(input_type)
-        horizontal_html = self._html_horizontal(input_type)
-        floating_html = self._html_floating(input_type)
+        default_html = (
+            '<div class="django_bootstrap5-req mb-3">'
+            '<label for="id_test" class="form-label">Test</label>'
+            f'<input class="form-control" id="id_test" name="test" placeholder="Test" required type="{input_type}">'
+            "</div>"
+        )
+        horizontal_html = (
+            '<div class="django_bootstrap5-req mb-3 row">'
+            '<label class="col-form-label col-sm-2" for="id_test">'
+            "Test"
+            '</label><div class="col-sm-10">'
+            f'<input class="form-control" id="id_test" name="test" placeholder="Test" required type="{input_type}">'
+            "</div>"
+            "</div>"
+        )
+        floating_html = (
+            '<div class="django_bootstrap5-req mb-3 form-floating">'
+            f'<input class="form-control" id="id_test" name="test" placeholder="Test" required type="{input_type}">'
+            '<label for="id_test" class="form-label">Test</label>'
+            "</div>"
+        )
 
         self.assertHTMLEqual(
             self.render("{% bootstrap_field form.test %}", context={"form": form}),
@@ -131,15 +158,33 @@ class InputTypeTextTestCase(BootstrapTestCase):
 
         self.assertHTMLEqual(
             self.render("{% bootstrap_field form.test %}", context={"form": form}),
-            self._html_default("password"),
+            (
+                '<div class="django_bootstrap5-req mb-3">'
+                '<label for="id_test" class="form-label">Test</label>'
+                '<input class="form-control" id="id_test" name="test" placeholder="Test" required type="password">'
+                "</div>"
+            ),
         )
 
         self.assertHTMLEqual(
             self.render("{% bootstrap_field form.test layout='horizontal' %}", context={"form": form}),
-            self._html_horizontal("password"),
+            (
+                '<div class="django_bootstrap5-req mb-3 row">'
+                '<label class="col-form-label col-sm-2" for="id_test">'
+                "Test"
+                '</label><div class="col-sm-10">'
+                '<input class="form-control" id="id_test" name="test" placeholder="Test" required type="password">'
+                "</div>"
+                "</div>"
+            ),
         ),
 
         self.assertHTMLEqual(
             self.render("{% bootstrap_field form.test layout='floating' %}", context={"form": form}),
-            self._html_floating("password"),
+            (
+                '<div class="django_bootstrap5-req mb-3 form-floating">'
+                '<input class="form-control" id="id_test" name="test" placeholder="Test" required type="password">'
+                '<label for="id_test" class="form-label">Test</label>'
+                "</div>"
+            ),
         )
